@@ -17,13 +17,12 @@ from arguebuf.models import Userdata
 from arguebuf.models.analyst import Analyst
 from arguebuf.models.edge import Edge
 from arguebuf.models.metadata import Metadata
-from arguebuf.models.node import (AtomNode, Attack, Node, Rephrase, SchemeNode,
-                                  Support)
+from arguebuf.models.node import AtomNode, Attack, Node, Rephrase, SchemeNode, Support
 from arguebuf.models.participant import Participant
 from arguebuf.models.reference import Reference
 from arguebuf.models.resource import Resource
-from arguebuf.schema import aif, ova
-from arguebuf.services import utils
+from arguebuf.schema import aif, ova, sadface
+from arguebuf.services import dt, utils
 from arguebuf.services.utils import ImmutableDict, ImmutableSet
 from google.protobuf.json_format import MessageToDict, ParseDict
 from lxml import html
@@ -709,6 +708,61 @@ class Graph:
 
         return g
 
+    @classmethod
+    def from_sadface(
+        cls,
+        obj: sadface.Graph,
+        name: t.Optional[str] = None,
+        atom_class: t.Type[AtomNode] = AtomNode,
+        scheme_class: t.Type[SchemeNode] = SchemeNode,
+        edge_class: t.Type[Edge] = Edge,
+        nlp: t.Optional[t.Callable[[str], t.Any]] = None,
+    ) -> Graph:
+        """Generate Graph structure from SADFace argument graph file
+        (reference: https://github.com/Open-Argumentation/SADFace/blob/master/examples/hangback/data.json).
+        """
+        g = cls(name)
+
+        for sadface_node in obj["nodes"]:
+            node = (
+                atom_class.from_sadface(sadface_node, nlp)
+                if sadface_node["type"] == "atom"
+                else scheme_class.from_sadface(sadface_node, nlp)
+            )
+
+            if node:
+                g.add_node(node)
+
+        for sadface_edge in obj["edges"]:
+            if edge := edge_class.from_sadface(sadface_edge, g._nodes):
+                g.add_edge(edge)
+
+        # create Metadata object
+        created = dt.from_format(
+            obj["metadata"]["core"]["created"], sadface.DATE_FORMAT
+        )
+        updated = dt.from_format(obj["metadata"]["core"]["edited"], sadface.DATE_FORMAT)
+        metadata = Metadata(created, updated)
+        g.metadata = metadata
+
+        # create Analyst object
+        analyst = Analyst(
+            name=obj["metadata"]["core"]["analyst_name"],
+            email=obj["metadata"]["core"]["analyst_email"],
+        )
+        g.add_analyst(analyst)
+
+        # create Userdata dict
+        userdata = {
+            "notes": obj["metadata"]["core"]["notes"],
+            "description": obj["metadata"]["core"]["description"],
+            "title": obj["metadata"]["core"]["title"],
+            "sadfaceVersion": obj["metadata"]["core"]["version"],
+        }
+        g.userdata = userdata
+
+        return g
+
     @classmethod
     def from_aif(
         cls,
@@ -1092,13 +1146,16 @@ class Graph:
     @classmethod
     def from_file(
         cls,
-        path: Path,
+        path: t.Union[Path, str],
         atom_class: t.Type[AtomNode] = AtomNode,
         scheme_class: t.Type[SchemeNode] = SchemeNode,
         edge_class: t.Type[Edge] = Edge,
         nlp: t.Optional[t.Callable[[str], t.Any]] = None,
     ) -> Graph:
         """Generate Graph structure from a File."""
+        if isinstance(path, str):
+            path = Path(path)
+
         with path.open("r", encoding="utf-8") as file:
             return cls.from_io(
                 file, path.suffix, path.stem, atom_class, scheme_class, edge_class, nlp
@@ -1106,11 +1163,14 @@ class Graph:
 
     def to_file(
         self,
-        path: Path,
+        path: t.Union[Path, str],
         format: GraphFormat = GraphFormat.ARGUEBUF,
         pretty: bool = False,
     ) -> None:
         """Export strucure of Graph instance into structure of File/Folder format."""
+        if isinstance(path, str):
+            path = Path(path)
+
         if path.is_dir() or not path.suffix:
             path = path / f"{self.name}.json"
 
@@ -1122,26 +1182,72 @@ class Graph:
     @classmethod
     def from_folder(
         cls,
-        path: Path,
+        path: t.Union[Path, str],
+        pattern: str,
         atom_class: t.Type[AtomNode] = AtomNode,
         scheme_class: t.Type[SchemeNode] = SchemeNode,
         edge_class: t.Type[Edge] = Edge,
         nlp: t.Optional[t.Callable[[str], t.Any]] = None,
-        suffixes: t.Iterable[str] = (".json"),
-    ) -> t.List[Graph]:
-        "Generate Graph structure from Folder."
-        return [
-            cls.from_file(file, atom_class, scheme_class, edge_class, nlp)
-            for suffix in suffixes
-            for file in sorted(path.rglob(f"*{suffix}"))
-        ]
+    ) -> t.Dict[Path, Graph]:
+        """Load all graphs matching the specified `pattern` in `path`.
+
+        Args:
+            path: Folder containing the graphs to be loaded.
+            pattern: Unix glob pattern to filter the available files.
+                Recursive matching can be achieved by prepending `**/` to any pattern.
+                For instance, all `json` files of a folder can be retrieved with `**/*.json`.
+                Supports the following wildcards: <https://docs.python.org/3/library/fnmatch.html#module-fnmatch>
+            atom_class: Allows to override the class used for atom nodes in case a specialized subclass has been created. Defaults to `AtomNode`.
+            scheme_class: Allows to override the class used for scheme nodes in case a specialized subclass has been created. Defaults to `SchemeNode`.
+            edge_class: Allows to override the class used for edges in case a specialized subclass has been created. Defaults to `Edge`.
+            nlp: Optionally pass a function to transforms all texts of atom nodes and resources to arbitrary Python objects.
+                Useful when using `spacy` to generate embeddings.
+                In this case, you can load a model with `spacy.load(...)` and pass the resulting `nlp` function via this parameter.
+
+        Returns:
+            Dictionary containing all found file paths as well as the loaded graphs.
+        """
+
+        if isinstance(path, str):
+            path = Path(path)
+
+        return {
+            file: cls.from_file(file, atom_class, scheme_class, edge_class, nlp)
+            for file in sorted(path.glob(pattern))
+        }
 
     def to_nx(
         self,
-        atom_label: t.Optional[t.Callable[[AtomNode], str]] = None,
-        scheme_label: t.Optional[t.Callable[[SchemeNode], str]] = None,
+        graph_attrs: t.Optional[
+            t.MutableMapping[str, t.Callable[[Graph], t.Any]]
+        ] = None,
+        atom_attrs: t.Optional[
+            t.MutableMapping[str, t.Callable[[AtomNode], t.Any]]
+        ] = None,
+        scheme_attrs: t.Optional[
+            t.MutableMapping[str, t.Callable[[SchemeNode], t.Any]]
+        ] = None,
+        edge_attrs: t.Optional[t.MutableMapping[str, t.Callable[[Edge], t.Any]]] = None,
     ) -> nx.DiGraph:
-        """Transform a Graph instance into an instance of networkx directed graph. Refer to the networkx library for additional information.
+        """Transform the argument graph for use with the library `NetworkX`
+
+        This library allows you to apply advanced graph-related algorithms directly on your argument graphs.
+        For instance, shortest paths and various distances can be computed.
+        [Documentation](https://networkx.org/documentation/stable/reference/index.html)
+
+        It is possible to add arbitrary attributes to the resulting graph and its elements.
+        For this, you need to pass a dictionary with the desired name of the attribute and a function that is used to compute the attribute's value.
+        The function will be passed the corresponding element as its only parameter.
+        For instance, you could pass `atom_attrs={"text": lambda node: node.plain_text}` to set a `text` attribute for atom nodes.
+
+        Args:
+            graph_attrs: Attribute functions for the whole graph.
+            atom_attrs: Attribute functions for the atom nodes.
+            scheme_attrs: Attribute functions for the scheme nodes.
+            edge_attrs: Attribute functions for the edges.
+
+        Returns:
+            Instance of the *directed* `NetworkX` graph.
 
         Examples:
             >>> g = Graph("Test")
@@ -1152,18 +1258,21 @@ class Graph:
             >>> gnx = g.to_nx()
             >>> gnx.number_of_nodes()
             2
-
         """
-        g = nx.DiGraph()
+
+        if graph_attrs is None:
+            graph_attrs = {}
+
+        g = nx.DiGraph(None, **{key: func(self) for key, func in graph_attrs.items()})
 
         for node in self._atom_nodes.values():
-            node.to_nx(g, atom_label)
+            node.to_nx(g, atom_attrs)
 
         for node in self._scheme_nodes.values():
-            node.to_nx(g, scheme_label)
+            node.to_nx(g, scheme_attrs)
 
         for edge in self._edges.values():
-            edge.to_nx(g)
+            edge.to_nx(g, edge_attrs)
 
         return g
 
@@ -1241,13 +1350,20 @@ class Graph:
 
         return g
 
-    def strip_snodes(self) -> None:
-        """Remove scheme nodes from graph and merge respective edges into singular edge"""
-        snodes = list(self._scheme_nodes.values())
+    to_dot = to_gv
 
-        for snode in snodes:
+    def strip_scheme_nodes(self) -> Graph:
+        """Remove scheme nodes from graph to connect atom nodes directly
+
+        Can be useful to analyze the structure of atom nodes
+        without considering their relation types (i.e., the scheme nodes between them).
+        """
+
+        schemes = list(self._scheme_nodes.values())
+
+        for scheme in schemes:
             for incoming, outgoing in itertools.product(
-                self._incoming_edges[snode], self._outgoing_edges[snode]
+                self._incoming_edges[scheme], self._outgoing_edges[scheme]
             ):
                 if isinstance(incoming.source, AtomNode) and isinstance(
                     outgoing.target, AtomNode
@@ -1256,26 +1372,36 @@ class Graph:
                         Edge(
                             incoming.source,
                             outgoing.target,
-                            id=f"{incoming.id}-{outgoing.id}",
                         )
                     )
 
-            self.remove_node(snode)
+            self.remove_node(scheme)
+
+        return self
 
     def copy(
         self,
         atom_class: t.Type[AtomNode] = AtomNode,
         scheme_class: t.Type[SchemeNode] = SchemeNode,
         edge_class: t.Type[Edge] = Edge,
+        participant_class: t.Type[Participant] = Participant,
+        analyst_class: t.Type[Analyst] = Analyst,
+        resource_class: t.Type[Resource] = Resource,
+        reference_class: t.Type[Reference] = Reference,
         nlp: t.Optional[t.Callable[[str], t.Any]] = None,
     ) -> Graph:
         """Contents of Graph instance are copied into new Graph object."""
-        return Graph.from_dict(
-            self.to_dict(format=GraphFormat.ARGUEBUF),
+
+        return Graph.from_protobuf(
+            self.to_protobuf(),
             self.name,
             atom_class,
             scheme_class,
             edge_class,
+            participant_class,
+            analyst_class,
+            resource_class,
+            reference_class,
             nlp,
         )
 
@@ -1325,22 +1451,28 @@ def _kialo_atom_node(
 
 def render(
     g: t.Union[gv.Graph, gv.Digraph],
-    path: Path,
+    path: t.Union[Path, str],
     view: bool = False,
 ) -> None:
     """Visualize a Graph instance using a GraphViz backend. Make sure that a GraphViz Executable path is set on your machine for visualization."""
+    if isinstance(path, str):
+        path = Path(path)
+
+    if not isinstance(g, (gv.Graph, gv.Digraph)):
+        raise ValueError(
+            "This method expects a graph in the 'DOT' format."
+            "Please use 'graph.to_gv()' to convert your argument graph to the 'DOT' format."
+        )
+
     filename = path.stem
     directory = path.parent
 
-    try:
-        g.render(
-            filename=filename,
-            directory=str(directory),
-            cleanup=True,
-            view=view,
-        )
-    except gv.ExecutableNotFound:
-        log.error("Rendering not possible. GraphViz might not be installed.")
+    g.render(
+        filename=filename,
+        directory=str(directory),
+        cleanup=True,
+        view=view,
+    )
 
 
 def _inject_original_text(
