@@ -15,7 +15,7 @@ from arguebuf.models.metadata import Metadata
 from arguebuf.models.participant import Participant
 from arguebuf.models.reference import Reference
 from arguebuf.models.resource import Resource
-from arguebuf.schema import aif, ova
+from arguebuf.schema import aif, ova, sadface
 from arguebuf.services import dt, utils
 
 NO_SCHEME_LABEL = "Unknown"
@@ -327,6 +327,15 @@ class Node(ABC):
     ) -> Node:
         """Generate Node object from AIF Node format."""
 
+    @classmethod
+    @abstractmethod
+    def from_sadface(
+        cls,
+        obj: sadface.Node,
+        nlp: t.Optional[t.Callable[[str], t.Any]] = None,
+    ) -> Node:
+        """Generate Node object from SADFace Node format."""
+
     @abstractmethod
     def to_aif(self) -> aif.Node:
         """Export Node object into AIF Node format."""
@@ -402,6 +411,21 @@ class AtomNode(Node):
     def __repr__(self):
         return utils.class_repr(self, [self._id, self.plain_text])
 
+    @classmethod
+    def from_sadface(
+        cls,
+        obj: sadface.Node,
+        nlp: t.Optional[t.Callable[[str], t.Any]] = None,
+    ) -> AtomNode:
+        """Generate AtomNode object from SADFace Node object."""
+        timestamp = pendulum.now()
+        return cls(
+            id=obj["id"],
+            text=utils.parse(obj["text"], nlp),
+            userdata=obj["metadata"],
+            metadata=Metadata(timestamp, timestamp),
+        )
+
     @classmethod
     def from_aif(
         cls,
@@ -479,13 +503,18 @@ class AtomNode(Node):
         return obj
 
     def to_nx(
-        self, g: nx.DiGraph, label_func: t.Optional[t.Callable[[AtomNode], str]] = None
+        self,
+        g: nx.DiGraph,
+        attrs: t.Optional[t.MutableMapping[str, t.Callable[[AtomNode], t.Any]]] = None,
     ) -> None:
         """Submethod used to export Graph object g into NX Graph format."""
-        g.add_node(
-            self._id,
-            label=label_func(self) if label_func else self.plain_text,
-        )
+        if attrs is None:
+            attrs = {}
+
+        if "label" not in attrs:
+            attrs["label"] = lambda x: x.plain_text
+
+        g.add_node(self._id, **{key: func(self) for key, func in attrs.items()})
 
     def color(self, major_claim: bool) -> Color:
         """Get the color for rendering the node."""
@@ -552,6 +581,31 @@ class SchemeNode(Node):
             ],
         )
 
+    @classmethod
+    def from_sadface(
+        cls,
+        obj: sadface.Node,
+        nlp: t.Optional[t.Callable[[str], t.Any]] = None,
+    ) -> SchemeNode:
+        """Generate SchemeNode object from SADFace Node object."""
+        name = None
+        if obj["name"] == "support":
+            name = Support
+        elif obj["name"] == "attack":
+            name = Attack
+        elif obj["name"] == "rephrase":
+            name = Rephrase
+        elif obj["name"] == "preference":
+            name = Preference
+
+        timestamp = pendulum.now()
+        return cls(
+            id=obj["id"],
+            userdata=obj["metadata"],
+            metadata=Metadata(timestamp, timestamp),
+            scheme=name,
+        )
+
     @classmethod
     def from_aif(
         cls,
@@ -685,23 +739,19 @@ class SchemeNode(Node):
     def to_nx(
         self,
         g: nx.DiGraph,
-        label_func: t.Optional[t.Callable[[SchemeNode], str]] = None,
+        attrs: t.Optional[
+            t.MutableMapping[str, t.Callable[[SchemeNode], t.Any]]
+        ] = None,
     ) -> None:
         """Submethod used to export Graph object g into NX Graph format."""
-        if label_func:
-            label = label_func(self)
-        elif self.scheme:
-            label = type(self.scheme).__name__
 
-            if self.scheme.value != "Default":
-                label = f"{label}: {self.scheme.value}"
-        else:
-            label = NO_SCHEME_LABEL
+        if attrs is None:
+            attrs = {}
 
-        g.add_node(
-            self._id,
-            label=label,
-        )
+        if "label" not in attrs:
+            attrs["label"] = _scheme_label
+
+        g.add_node(self._id, **{key: func(self) for key, func in attrs.items()})
 
     def color(self, major_claim: bool) -> Color:
         """Get the color used in OVA based on `category`."""
@@ -720,17 +770,9 @@ class SchemeNode(Node):
         label_func: t.Optional[t.Callable[[SchemeNode], str]] = None,
     ) -> None:
         """Submethod used to export Graph object g into GV Graph format."""
+
         color = self.color(major_claim)
-
-        if label_func:
-            label = label_func(self)
-        elif self.scheme:
-            label = type(self.scheme).__name__
-
-            if self.scheme.value != "Default":
-                label = f"{label}: {self.scheme.value}"
-        else:
-            label = NO_SCHEME_LABEL
+        label = label_func(self) if label_func else _scheme_label(self)
 
         g.node(
             self._id,
@@ -739,3 +781,15 @@ class SchemeNode(Node):
             fillcolor=color.bg,
             color=color.border,
         )
+
+
+def _scheme_label(obj: SchemeNode) -> str:
+    label = NO_SCHEME_LABEL
+
+    if obj.scheme:
+        label = type(obj.scheme).__name__
+
+        if obj.scheme.value != "Default":
+            label = f"{label}: {obj.scheme.value}"
+
+    return label
